import { Component, OnInit, Inject } from '@angular/core';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material';
import { DataService } from '../../../core/data.service';
import { FormBuilder, FormGroup } from '@angular/forms';

@Component({
	selector: 'cse-notes-dialog',
	templateUrl: './notes-dialog.component.html',
	styleUrls: ['./notes-dialog.component.scss']
})

export class NotesDialogComponent implements OnInit {
	noteForm: FormGroup;
	id: number;
	note: string;
	termCode: string;
	noteData: object;

	// tslint:disable-next-line:max-line-length
	constructor(private dataService: DataService, private fb: FormBuilder, private dialogRef: MatDialogRef<NotesDialogComponent>, @Inject(MAT_DIALOG_DATA) data) {
		this.id = data.id;
		this.termCode = data.termCode;
		this.note = data.note;
		this.noteData = data;
	}

	ngOnInit() {
		this.noteForm = this.fb.group({
			id: [this.id, []],
			termCode: [this.termCode, []],
			note: [this.note, []],
		});
	}

	removeNote() {
		this.dataService.removeNote(this.id)
		.subscribe(result => {
			this.dialogRef.close();
		});
	}

	saveNote() {
		const note = this.noteForm.value;

		if (note.id) {
			this.dataService.updateNote(note)
			.subscribe(result => {
				this.dialogRef.close();
			});
		} else {
			this.dataService.createNote(note)
			.subscribe(result => {
				this.dialogRef.close();
			});
		}
	}
}
